/*

    pHash, the open source perceptual hash library
    Copyright (C) 2008-2009 Aetilius, Inc.
    All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Christoph Zauner - christoph.zauner@students.fh-hagenberg.at

	A class representing a binary hash of arbitrary byte length.
	That is, a hash consisting of only ones and zeroes. E.g.
	{10101000} (binary hash with length of 8).

*/

#ifndef _BINHASH_H
#define _BINHASH_H

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "pHashReturnVals.h"

class BinHash {

	public:

	BinHash(uint32_t bytelength)
	: bytelength(bytelength)
	{
		hash = (uint8_t*)calloc(sizeof(uint8_t), bytelength);
		byteidx = 0;
		bitmask = 128;
	}

	uint8_t *hash;
	uint32_t bytelength;
	uint32_t byteidx; // used by addbit()
	uint8_t bitmask;  // used by addbit()

	/*
	 * add a single bit to hash. the bits are 
	 * written from left to right.
	 */
	PHRetCode addbit(uint8_t bit)
	{
		if (bitmask == 0) 
		{
			bitmask = 128; // reset bitmask to "10000000"
			byteidx++;     // jump to next byte in array
		}

		if (byteidx >= bytelength) return PHO_ERROR;
		
		if (bit == 1) *(hash + byteidx) |= bitmask;
		bitmask >>=1;
		return PHO_SUCCESS;
	}		

	/*
	 * print the hash in human readable form to stdout.
	 */
	void print()
	{
		printf("hash value: >");
		for (int i = 0; i < bytelength; i++)
		{
			printf("%s", byte_to_binarystring(*(hash + i)));
		}
		printf("<\n");
	}

	/* 
	 * converts a byte to a binary coded string.
	 */
	char* byte_to_binarystring(uint8_t byte)
	{
		static char binary[9];
		binary[8] = 0;
		for (int i = 7; i >= 0; i--, byte >>= 1) {
			if (byte & 1)
			{
				binary[i] = '1';
			} else {
				binary[i] = '0';
			}
		}
		return binary;
	}

	/*
	 * set the hash
	 */
	void set_hash(uint8_t *hash)
	{
		memcpy(this->hash, hash, bytelength);
	}
};

#endif // BinHash.h
